'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactUpdate = require('react-update');

var _reactUpdate2 = _interopRequireDefault(_reactUpdate);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _xhr = require('../../xhr');

var _xhr2 = _interopRequireDefault(_xhr);

require('./index.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright 2016-present, Baifendian, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

var Form = (function (_Component) {
  (0, _inherits3.default)(Form, _Component);

  function Form(props) {
    (0, _classCallCheck3.default)(this, Form);

    // 存储 FormItem 实例，用于访问 FormItem

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(Form).call(this));

    _this.items = [];
    _this.state = {
      data: props.data || props.defaultData || {}
    };
    return _this;
  }

  (0, _createClass3.default)(Form, [{
    key: 'getChildContext',
    value: function getChildContext() {
      return {
        form: this
      };
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(_ref) {
      var data = _ref.data;

      data && this.setState({ data: data });
    }
  }, {
    key: 'getItemValue',
    value: function getItemValue(instance, props) {
      var _ref2 = props || instance.props;

      var name = _ref2.name;
      var multiple = _ref2.multiple;

      var value = this.state.data[name];
      if (!multiple) {
        return value;
      } else {
        if (value instanceof Array) {
          return value[this.multipleMap[instance.uuid]];
        }
        return null;
      }
    }
  }, {
    key: 'setItemValue',
    value: function setItemValue(instance, value) {
      var _instance$props = instance.props;
      var name = _instance$props.name;
      var multiple = _instance$props.multiple;

      var data = this.state.data;
      var nextData = undefined;
      if (!multiple) {
        nextData = (0, _reactUpdate2.default)(data, 'set', name, value);
      } else {
        if (!data[name]) {
          data[name] = [];
        }
        nextData = (0, _reactUpdate2.default)(data, 'set', [name, this.multipleMap[instance.uuid]], value);
      }
      this.setState({ data: nextData });
      this.props.onChange && this.props.onChange(nextData);
    }
  }, {
    key: 'addItem',
    value: function addItem(newItem) {
      if (newItem.props.multiple) {
        newItem.uuid = Math.random().toString(16).slice(2);
        this.multipleMap || (this.multipleMap = {});
        this.multipleMap[newItem.uuid] = this.items.filter(function (item) {
          return item.props.name === newItem.props.name;
        }).length;
      }
      this.items.push(newItem);
    }
  }, {
    key: 'removeItem',
    value: function removeItem(item) {
      var _this2 = this;

      if (item.props.multiple) {
        (function () {
          delete _this2.multipleMap[item.uuid];
          // Reset index
          var name = item.props.name;
          _this2.items.filter(function (item) {
            return item.props.name === name;
          }).forEach(function (item, i) {
            _this2.multipleMap[item.uuid] = i;
          });
        })();
      }
      this.items.splice(this.items.indexOf(item), 1);
    }

    /**
     * @public
     * @name this.refs.form.validate
     * @param  {object} [data] 验证的数据，默认验证表单当前的数据
     * @return {boolean} 成功/失败
     * @description 表单整体验证
     */

  }, {
    key: 'validate',
    value: function validate(data) {
      var _this3 = this;

      data || (data = this.state.data);
      var isValid = true;
      this.items.forEach(function (formItem) {
        var _formItem$props = formItem.props;
        var name = _formItem$props.name;
        var multiple = _formItem$props.multiple;

        if (!multiple) {
          formItem.validate(data[name]) || (isValid = false);
        } else {
          var index = _this3.multipleMap[formItem.uuid];
          formItem.validate(data[name] && data[name][index]) || (isValid = false);
        }
      });
      return isValid;
    }

    /**
     * @public
     * @name this.refs.form.save
     * @param  {object} [data] 提交的数据，默认提交表单当前的数据
     * @description 表单提交，提交地址为 action 属性, 默认提交 data
     * 属性数据，可以自定义传入。发送请求前会进行表单验证, 提交成功会响应 onSuccess
     */

  }, {
    key: 'save',
    value: function save() {
      var _this4 = this;

      var data = arguments.length <= 0 || arguments[0] === undefined ? this.state.data : arguments[0];
      var _props = this.props;
      var action = _props.action;
      var sendDataFormatter = _props.sendDataFormatter;
      var onSuccess = _props.onSuccess;

      if (this.validate()) {
        (0, _invariant2.default)(action, 'No `action` provided, check the Form component you save.');
        this.submit && this.submit.toggleProcess(true);
        (0, _xhr2.default)({
          type: 'POST',
          url: action,
          data: sendDataFormatter ? sendDataFormatter(data) : data,
          success: function success(data) {
            onSuccess && onSuccess(data);
          },
          complete: function complete() {
            _this4.submit && _this4.submit.toggleProcess(false);
          }
        });
      }
    }
  }, {
    key: 'handleSubmit',
    value: function handleSubmit(e) {
      e.preventDefault();
      this.props.onSubmit && this.props.onSubmit(this.state.data);
    }
  }, {
    key: 'render',
    value: function render() {
      var _props2 = this.props;
      var children = _props2.children;
      var className = _props2.className;
      var data = _props2.data;
      var defaultData = _props2.defaultData;
      var onChange = _props2.onChange;
      var onSubmit = _props2.onSubmit;
      var onSuccess = _props2.onSuccess;
      var rules = _props2.rules;
      var labelWidth = _props2.labelWidth;
      var other = (0, _objectWithoutProperties3.default)(_props2, ['children', 'className', 'data', 'defaultData', 'onChange', 'onSubmit', 'onSuccess', 'rules', 'labelWidth']);

      return _react2.default.createElement(
        'form',
        (0, _extends3.default)({
          onSubmit: this.handleSubmit.bind(this),
          className: (0, _classnames2.default)('bfd-form', className)
        }, other),
        children
      );
    }
  }]);
  return Form;
})(_react.Component);

Form.childContextTypes = {
  form: _react.PropTypes.instanceOf(Form)
};

Form.defaultProps = {
  labelWidth: 100
};

Form.propTypes = {

  // 表单数据源，作为表单验证时的数据对象、表单提交的数据源、FormInput、FormSelect、FormTextarea 等自动填充的数据源
  data: _react.PropTypes.object,

  // 表单数据源（不可控），同 data
  defaultData: _react.PropTypes.object,

  // 表单数据改变后的回调，参数为当前表单的数据
  onChange: _react.PropTypes.func,

  // 表单验证规则，需要验证的 key 与 data 对象一一对应
  rules: _react.PropTypes.object,

  // 表单 label 宽度，如果不需要 label，设置为 0 即可
  labelWidth: _react.PropTypes.number,

  // 表单提交 URL，内部调用 xhr 模块
  action: _react.PropTypes.string,

  // 成功后的回调，参数为服务器返回后的数据
  onSuccess: _react.PropTypes.func,

  // 表单提交回调，参数为当前表单的数据，自定义提交后的行为，不会自动调用
  // save。不指定此属性，表单提交后自动调用 save
  onSubmit: _react.PropTypes.func,

  // 提交的数据处理器，参数为当前表单数据，返回最终发送的数据
  sendDataFormatter: _react.PropTypes.func,

  customProp: function customProp(_ref3) {
    var data = _ref3.data;
    var onChange = _ref3.onChange;

    if (data && !onChange) {
      return new Error('You provided a `data` prop without an `onChange` handler');
    }
  }
};

exports.default = Form;