'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright 2016-present, Baifendian, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

function getDocumentScroll() {
  var _document = document;
  var documentElement = _document.documentElement;
  var body = _document.body;

  return [documentElement && documentElement.scrollTop || body.scrollTop, documentElement && documentElement.scrollLeft || body.scrollLeft];
}

function CoordinateFactory(triggerNode, popoverNode, direction, align) {
  var _getDocumentScroll = getDocumentScroll();

  var _getDocumentScroll2 = (0, _slicedToArray3.default)(_getDocumentScroll, 2);

  var scrollTop = _getDocumentScroll2[0];
  var scrollLeft = _getDocumentScroll2[1];

  var CF = CoordinateFactory;

  CF.triggerRect = triggerNode.getBoundingClientRect();
  CF.popoverRect = popoverNode.getBoundingClientRect();
  CF.direction = CF.getComputedDirection(direction);
  CF.align = CF.getComputedAlign(align);
  CF.center = [CF.triggerRect.left + CF.triggerRect.width / 2 + scrollLeft, CF.triggerRect.top + CF.triggerRect.height / 2 + scrollTop];

  var _CF$coordinateMap$CF$ = CF.coordinateMap[CF.direction].call(CF);

  var _CF$coordinateMap$CF$2 = (0, _slicedToArray3.default)(_CF$coordinateMap$CF$, 2);

  var left = _CF$coordinateMap$CF$2[0];
  var top = _CF$coordinateMap$CF$2[1];

  popoverNode.style.left = left + 'px';
  popoverNode.style.top = top + 'px';

  return [CF.direction, CF.align];
}

(0, _assign2.default)(CoordinateFactory, {

  triggerRect: null,

  popoverRect: null,

  align: null,

  center: null,

  getComputedDirection: function getComputedDirection(direction) {
    var triggerRect = this.triggerRect;
    var popoverRect = this.popoverRect;

    if (direction === 'up' || direction === 'down') {
      if (triggerRect.top < popoverRect.height) {
        direction = 'down';
      } else if (popoverRect.height + triggerRect.top + triggerRect.height > window.innerHeight) {
        direction = 'up';
      }
    }
    return direction;
  },
  getComputedAlign: function getComputedAlign(align) {
    if (align === 'middle') {
      return align;
    }
    if (this.direction === 'up' || this.direction === 'down') {
      if (align === 'top' || align === 'bottom') {
        return 'middle';
      }
    } else {
      if (align === 'left' || align === 'right') {
        return 'middle';
      }
    }
    return align;
  },
  getAlignPosition: function getAlignPosition(horizontal) {
    var types = horizontal ? ['left', 'right'] : ['top', 'bottom'];
    var axis = horizontal ? 0 : 1;
    var size = horizontal ? 'width' : 'height';
    if (this.align === types[0]) {
      return this.center[axis] - this.triggerRect[size] / 2;
    } else if (this.align === types[1]) {
      return this.center[axis] - this.popoverRect[size] + this.triggerRect[size] / 2;
    } else {
      return this.center[axis] - this.popoverRect[size] / 2;
    }
  },

  coordinateMap: {
    up: function up() {
      var top = this.center[1] - this.triggerRect.height / 2 - this.popoverRect.height;
      return [this.getAlignPosition(true), top];
    },
    down: function down() {
      var top = this.center[1] + this.triggerRect.height / 2;
      return [this.getAlignPosition(true), top];
    },
    left: function left() {
      var left = this.center[0] - this.triggerRect.width / 2 - this.popoverRect.width;
      return [left, this.getAlignPosition()];
    },
    right: function right() {
      var left = this.center[0] + this.triggerRect.width / 2;
      return [left, this.getAlignPosition()];
    }
  }
});

exports.default = CoordinateFactory;