'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _class = (function () {
  (0, _createClass3.default)(_class, [{
    key: 'getDomain',

    /**
     * 获取Y轴最大最小值区间
     */
    value: function getDomain(series) {
      var domain = [];
      series.forEach(function (serie) {
        domain = domain.concat(_d2.default.extent(serie.data, function (d) {
          return d;
        }));
      });
      domain = _d2.default.extent(domain, function (d) {
        return d;
      });

      var paddingScale = .2;
      var padding = (domain[1] - domain[0]) * paddingScale;
      domain = [domain[0] - padding, domain[1] + padding];
      return domain;
    }
  }]);

  function _class(config) {
    (0, _classCallCheck3.default)(this, _class);
    var container = config.container;
    var category = config.category;
    var cols = config.cols;
    var data = config.data;

    var yAxisConfig = config.yAxis || {};
    var colors = ['#80deea', '#9fa8da'];

    var margin = {
      top: 20,
      right: 20,
      bottom: 30,
      left: 40
    };

    // 实例唯一ID
    var id = Math.random().toString(16).slice(2);

    var _w = container.clientWidth;
    var _h = container.clientHeight || _w * .5;

    var width = _w - margin.left - margin.right;
    var height = _h - margin.top - margin.bottom;

    var svg = _d2.default.select(container).html('').append('svg').attr('width', _w).attr('height', _h).append('g').attr('transform', 'translate(' + margin.left + ', ' + margin.top + ')');

    // get data
    var series = (0, _keys2.default)(cols).map(function (key) {
      return {
        key: key, name: cols[key], data: []
      };
    });

    var categories = [];

    data.forEach(function (item) {
      categories.push(item[category]);
      series.forEach(function (serie) {
        serie.data.push(item[serie.key]);
      });
    });

    var xAxisPaddingScale = 20 / (width / categories.length);
    var xScale = _d2.default.scale.ordinal().rangePoints([0, width], xAxisPaddingScale).domain(categories);

    var yScale = _d2.default.scale.linear().range([height, 0]).domain(this.getDomain(series));

    var xAxis = _d2.default.svg.axis().scale(xScale).orient('bottom').tickSize(6, 0);

    // 处理x轴数据节点过多的问题
    var interval = Math.ceil(categories.join('').length / (width / 10));
    interval > 1 && xAxis.tickValues(xScale.domain().filter(function (d, i) {
      return !(i % interval);
    }));

    var yAxis = _d2.default.svg.axis().scale(yScale).orient('left').ticks(5).tickFormat(function (d) {
      return _d2.default.format(yAxisConfig.format || 's')(d);
    }).tickSize(-width);

    svg.append('g').attr('class', 'axis-x').attr('transform', 'translate(0, ' + height + ')').call(xAxis);

    svg.append('g').attr('class', 'axis-y').call(yAxis);

    // 初始化渲染动画，从左到右划出
    svg.append('clipPath').attr('id', 'rectClip-' + id).append('rect').attr('width', 0).attr('height', height + 10).transition().duration(900).attr('width', width);

    /**
     * Tooltip
     */
    var tooltip = _d2.default.select(container).append('div').attr('class', 'tooltip').style({
      position: 'absolute',
      left: 0,
      top: 0,
      opacity: 0,
      'pointer-events': 'none'
    });

    var seriesGroup = svg.append('g').attr('clip-path', 'url(#rectClip-' + id + ')');
    series.forEach(function (serie, i) {
      seriesGroup.append('g').selectAll('circle').data(data).enter().append('circle').attr('r', 6).attr('cx', function (d) {
        return xScale(d[category]);
      }).attr('cy', function (d) {
        return yScale(d[serie.key]);
      }).attr('fill', colors[i]).attr('fill-opacity', .7).attr('name', serie.name).attr('value', function (d, i) {
        return serie.data[i];
      }).on('mouseenter', function () {
        var target = _d2.default.select(this);
        target.attr('r', 10);

        // Tooltip content
        var html = categories[i] + '<br>' + target.attr('name') + '：' + target.attr('value');
        tooltip.html(html).style({
          left: _d2.default.event.offsetX + 'px',
          top: _d2.default.event.offsetY + 'px',
          opacity: 1
        });
      }).on('mouseout', function () {
        var target = _d2.default.select(this);
        target.attr('r', 10);

        target.attr('r', 6);

        tooltip
        // .delay(200)
        .style('opacity', 0);
      });
    });
  }

  return _class;
})(); /**
       * Copyright 2016-present, Baifendian, Inc.
       * All rights reserved.
       *
       * This source code is licensed under the BSD-style license found in the
       * LICENSE file in the root directory of this source tree. An additional grant
       * of patent rights can be found in the PATENTS file in the same directory.
       */

exports.default = _class;