'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _Modal = require('../Modal');

var _Button = require('../Button');

var _Button2 = _interopRequireDefault(_Button);

require('./index.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright 2016-present, Baifendian, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

var Confirm = (function (_Component) {
  (0, _inherits3.default)(Confirm, _Component);

  function Confirm() {
    (0, _classCallCheck3.default)(this, Confirm);
    return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(Confirm).apply(this, arguments));
  }

  (0, _createClass3.default)(Confirm, [{
    key: 'handleConfirm',
    value: function handleConfirm() {
      this.props.onConfirm && this.props.onConfirm();
      this.close();
    }
  }, {
    key: 'open',
    value: function open() {
      this.refs.modal.open();
    }
  }, {
    key: 'close',
    value: function close() {
      this.refs.modal.close();
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var title = _props.title;
      var content = _props.content;
      var operation = _props.operation;
      var okText = _props.okText;
      var cancelText = _props.cancelText;

      return _react2.default.createElement(
        _Modal.Modal,
        { className: 'bfd-confirm', ref: 'modal' },
        _react2.default.createElement(
          _Modal.ModalHeader,
          null,
          _react2.default.createElement(
            'h4',
            null,
            title
          )
        ),
        _react2.default.createElement(
          _Modal.ModalBody,
          null,
          _react2.default.createElement(
            'div',
            { className: 'bfd-confirm__message' },
            content
          ),
          _react2.default.createElement(
            'div',
            { className: 'bfd-confirm__operate' },
            operation ? operation : _react2.default.createElement(
              'div',
              null,
              _react2.default.createElement(
                _Button2.default,
                { onClick: this.handleConfirm.bind(this) },
                okText
              ),
              _react2.default.createElement(
                _Button2.default,
                { type: 'minor', onClick: this.close.bind(this) },
                cancelText
              )
            )
          )
        )
      );
    }
  }]);
  return Confirm;
})(_react.Component);

Confirm.defaultProps = {
  title: '确认提示',
  okText: '确定',
  cancelText: '取消'
};

Confirm.propTypes = {
  title: _react.PropTypes.node,
  okText: _react.PropTypes.node,
  cancelText: _react.PropTypes.node,
  content: _react.PropTypes.node,
  operation: _react.PropTypes.element,
  onConfirm: _react.PropTypes.func
};

/**
 * @public
 * @name confirm
 * @param  { ReactNode | Object } content 显示内容 | 配置项
 * @param  { function } [onConfirm] 确定后的回调
 * @description 确认提示，eg
 * ```js
 * confirm('确认删除吗？', () => console.log('确认'))
 * ```
 * 更多控制请使用配置项方式：
 * ```js
 * confirm({
 *   content: 'xxx', // 提示内容
 *   onConfirm: function() {}, // 确定后的回调，非自定义操作下使用
 *   operation: <Button>xxx</Button>, // 自定义操作逻辑，默认两个按钮：确认、取消
 *   title: 'xxx', // 提示框标题，默认`确认提示`，可全局配置
 *   okText: 'xxx', // 确定按钮文字，默认`确定`，可全局配置
 *   cancelText: 'xxx', // 取消按钮文字，默认`取消`，可全局配置
 * })
 * ```
 */

var instance = undefined,
    container = undefined;

function confirm(content, onConfirm) {

  var props = undefined;
  if ((0, _isPlainObject2.default)(content) && !_react2.default.isValidElement(content)) {
    props = content;
  } else {
    props = { content: content, onConfirm: onConfirm };
  }
  if (!container) {
    container = document.createElement('div');
  }
  // Model 会单独创建真实 DOM 容器，所以 Confirm 的容器无需挂在到真实 DOM 中
  instance = (0, _reactDom.render)(_react2.default.createElement(Confirm, props), container);
  instance.open();
}

/**
 * @public
 * @name confirm.close
 * @description 关闭确认提示
 */
confirm.close = function () {
  instance.close();
};

/**
 * @public
 * @name confirm.config
 * @param { Object } options
 * @description confirm 全局配置，配置项
 * ```js
 * {
 *   title: 'xxx', // 提示框标题，默认`确认提示`
 *   okText: 'xxx', // 确定按钮文字，默认`确定`
 *   cancelText: 'xxx', // 取消按钮文字，默认`取消`
 * }
 * ```
 */
confirm.config = function (options) {
  (0, _assign2.default)(Confirm.defaultProps, options);
};

exports.default = confirm;